// -*- C++ -*-

//
//  kjots
//
//  Copyright (C) 1997 Christoph Neerfeld <Christoph.Neerfeld@home.ivm.de>
//  Copyright (C) 2002, 2003 Aaron J. Seigo <aseigo@kde.org>
//  Copyright (C) 2003 Stanislav Kljuhhin <crz@hot.ee>
//  Copyright (C) 2005-2006 Jaison Lee <lee.jaison@gmail.com>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#include "../config.h"

#include <stdlib.h>
#include <stdio.h>
#include <kuniqueapplication.h>
#include <kcmdlineargs.h>
#include <kconfig.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <kglobal.h>
#include "KJotsMain.h"


static const char description[] = I18N_NOOP("KDE note taking utility");

int main( int argc, char **argv )
{
    KAboutData aboutData( "kjots", I18N_NOOP("KJots"),
        KDE_VERSION_STRING, description, KAboutData::License_GPL,
        "(c) 1997-2002, Christoph Neerfeld");
    aboutData.addAuthor("Jaison Lee", I18N_NOOP("Current maintainer"), "lee.jaison@gmail.com");
    aboutData.addAuthor("Aaron J. Seigo", 0, "aseigo@kde.org");
    aboutData.addAuthor("Stanislav Kljuhhin", 0, "crz@starman.ee");
    aboutData.addAuthor("Christoph Neerfeld", I18N_NOOP("Original author"), "chris@kde.org");
    KCmdLineArgs::init(argc, argv, &aboutData);
    KUniqueApplication::addCmdLineOptions();

    if (!KUniqueApplication::start()) {
        fprintf(stderr, "kjots is already running!\n");
        exit(0);
    }

    KUniqueApplication a;

    // backwards compatibility code to convert "old" user font settings
    // to the new config settings
    KSharedConfig::Ptr config = KGlobal::config();
    config->setGroup("kjots");
    if (config->hasKey("EFontFamily")) {
      // read old font and create it
      QFont font( config->readEntry("EFontFamily"),
                  config->readEntry("EFontSize", 12),
                  config->readEntry("EFontWeight", 0),
                  config->readEntry("EFontItalic", 0));
      // delete old entries
      config->deleteEntry("EFontFamily");
      config->deleteEntry("EFontSize");
      config->deleteEntry("EFontWeight");
      config->deleteEntry("EFontItalic");
      config->deleteEntry("EFontCharset");
      // write new "converted" entry
      config->writeEntry("Font", font);
    }

    KJotsMain *jots = new KJotsMain;
    if( a.isSessionRestored() ) {
        if( KJotsMain::canBeRestored(1) ) {
            jots->restore(1);
        }
    }
  
    jots->show();
    jots->resize(jots->size());
    return a.exec();
}

/* ex: set tabstop=4 softtabstop=4 shiftwidth=4 expandtab: */
/* kate: tab-indents off; replace-tabs on; tab-width 4; remove-trailing-space on; encoding utf-8;*/
